<?php

namespace Drupal\Tests\yaml_translation_import\Functional;

use Drupal\Core\Url;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the yaml translation import from the UI.
 *
 * @group yaml_translation_import
 */
class ImportFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['yaml_translation_import'];

  /**
   * User with admin rights.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $adminUser;

  /**
   * The relative path to the current module.
   *
   * @var string
   */
  protected $relativePathToModule;

  /**
   * A regular user.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $webUser;

  /**
   * The locale storage.
   *
   * @var \Drupal\locale\StringStorageInterface
   */
  protected $localeStorage;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $this->webUser = $this->createUser();
    $this->adminUser = $this->createUser(['access yaml import page']);

    ConfigurableLanguage::createFromLangcode('nl')->save();
    ConfigurableLanguage::createFromLangcode('fr')->save();

    $this->relativePathToModule = $this->container->get('extension.list.module')->getPath('yaml_translation_import');
    $this->localeStorage = \Drupal::service('locale.storage');
  }

  /**
   * Tests the yaml translation import from the UI.
   */
  public function testYamlTranslationsSettingsForm() {
    $assert_session = $this->assertSession();
    $fr_language = ConfigurableLanguage::load('fr');
    $nl_language = ConfigurableLanguage::load('nl');

    // Unauthorized user should not have access.
    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->assertResponse(403);

    // Login as a regular user.
    $this->drupalLogin($this->webUser);

    // Unauthorized user should not have access.
    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->assertResponse(403);

    // Login as an admin user.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->assertResponse(200);

    // Check that the enabled languages are present.
    $assert_session->elementExists('css', 'input[name="languages[nl]"]');
    $assert_session->elementExists('css', 'input[name="languages[fr]"]');
    $assert_session->elementExists('css', 'input[name="languages[fr]"]');
    $assert_session->elementNotExists('css', 'input[name="languages[de]"]');

    $this->drupalPostForm(NULL, [
      'languages[nl]' => TRUE,
      'languages[fr]' => TRUE,
    ], 'Import');

    $assert_session->pageTextContains(sprintf('Import for "%s" was skipped because the translation file was not found', $fr_language->label()));
    $assert_session->pageTextContains(sprintf('Import for "%s" was skipped because the translation file was not found', $nl_language->label()));

    // Update the path so this time there will be an import file.
    $config = $this->config('yaml_translation_import.settings');
    $config->set('translation.directory', $this->relativePathToModule . '/tests/fixtures/translation_files/v1');
    $config->save();

    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->drupalPostForm(NULL, [
      'languages[nl]' => TRUE,
      'languages[fr]' => TRUE,
    ], 'Import');
    $assert_session->pageTextContains(sprintf('Import for "%s" was skipped because the translation file was not found', $fr_language->label()));
    $assert_session->pageTextContains(sprintf('%s translations imported: %s added, %s updated, %s removed.', $nl_language->label(), 1, 0, 0));
    $this->checkIfTranslationExists('nl', 'nl.test1', 'NL test 1');
    $this->checkIfTranslationNotExists('nl', 'nl.test2');
    $this->checkIfTranslationNotExists('fr', 'fr.test1');

    // Update the path so this time there will be an import file.
    $config = $this->config('yaml_translation_import.settings');
    $config->set('translation.directory', $this->relativePathToModule . '/tests/fixtures/translation_files/v2');
    $config->save();

    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->drupalPostForm(NULL, [
      'languages[nl]' => TRUE,
      'languages[fr]' => TRUE,
    ], 'Import');
    $assert_session->pageTextContains(sprintf('%s translations imported: %s added, %s updated, %s removed.', $fr_language->label(), 2, 0, 0));
    $assert_session->pageTextContains(sprintf('%s translations imported: %s added, %s updated, %s removed.', $nl_language->label(), 0, 0, 0));
    $this->checkIfTranslationExists('fr', 'fr.test1', 'FR test 1');
    $this->checkIfTranslationExists('fr', 'fr.test2', 'FR test 2');
    $this->checkIfTranslationNotExists('nl', 'nl.test2');

    // Update the path so this time there will be an import file.
    $config = $this->config('yaml_translation_import.settings');
    $config->set('translation.directory', $this->relativePathToModule . '/tests/fixtures/translation_files/v3');
    $config->save();

    $this->drupalGet(Url::fromRoute('yaml_translation_import.yaml.import'));
    $this->drupalPostForm(NULL, [
      'languages[nl]' => TRUE,
      'languages[fr]' => TRUE,
      'overwrite_customized' => TRUE,
    ], 'Import');
    $assert_session->pageTextContains(sprintf('%s translations imported: %s added, %s updated, %s removed.', $fr_language->label(), 0, 2, 0));
    $assert_session->pageTextContains(sprintf('%s translations imported: %s added, %s updated, %s removed.', $nl_language->label(), 1, 1, 0));
    $this->checkIfTranslationExists('nl', 'nl.test1', 'NL new test 1');
    $this->checkIfTranslationExists('nl', 'nl.test2', 'NL new test 2');
    $this->checkIfTranslationExists('fr', 'fr.test1', 'FR new test 1');
    $this->checkIfTranslationExists('fr', 'fr.test2', 'FR new test 2');
  }

  /**
   * Check if a translation exists.
   *
   * @param string $langcode
   *   The langcode we're searching.
   * @param string $source
   *   The translation source.
   * @param string $translation
   *   The actual translation.
   */
  protected function checkIfTranslationExists(string $langcode, string $source, string $translation): void {
    $translation_strings = $this->localeStorage->getTranslations([
      'language' => $langcode,
      'source' => $source,
    ]);
    $translation_strings = reset($translation_strings);

    $this->assertEquals($source, $translation_strings->source);
    $this->assertEquals($translation, $translation_strings->getString());
  }

  /**
   * Check if a translation NOT exists.
   *
   * @param string $langcode
   *   The langcode we're searching.
   * @param string $source
   *   The translation source.
   */
  protected function checkIfTranslationNotExists(string $langcode, string $source): void {
    $translation_strings = $this->localeStorage->getTranslations([
      'language' => $langcode,
      'source' => $source,
    ]);

    $this->assertEmpty($translation_strings);
  }

}
